;; -*- Mode: Scheme -*-
;;   This file contains instructions that support fixed-point operations
;;   for ATMEL AVR micro controllers.
;;   Copyright (C) 2009
;;   Free Software Foundation, Inc.
;;   Contributed by Sean D'Epagnier (sean@depagnier.com)

;; This file is part of GCC.

;; GCC is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3, or (at your option)
;; any later version.

;; GCC is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GCC; see the file COPYING3.  If not see
;; <http://www.gnu.org/licenses/>.

(define_mode_iterator ALLQQ [(QQ "") (UQQ "")])
(define_mode_iterator ALLHQ [(HQ "") (UHQ "")])
(define_mode_iterator ALLHA [(HA "") (UHA "")])
(define_mode_iterator ALLHQHA [(HQ "") (UHQ "") (HA "") (UHA "")])
(define_mode_iterator ALLSA [(SA "") (USA "")])

;;; Conversions

(define_mode_iterator FIXED1 [(QQ "") (UQQ "") (HQ "") (UHQ "")
                              (SQ "") (USQ "") (DQ "") (UDQ "")
                              (HA "") (UHA "") (SA "") (USA "")
                              (DA "") (UDA "") (TA "") (UTA "")
                              (QI "") (HI "") (SI "") (DI "")])
(define_mode_iterator FIXED2 [(QQ "") (UQQ "") (HQ "") (UHQ "")
                              (SQ "") (USQ "") (DQ "") (UDQ "")
                              (HA "") (UHA "") (SA "") (USA "")
                              (DA "") (UDA "") (TA "") (UTA "")
                              (QI "") (HI "") (SI "") (DI "")])

(define_insn "fract<FIXED2:mode><FIXED1:mode>2"
  [(set (match_operand:FIXED1 0 "register_operand" "=r")
        (fract_convert:FIXED1 (match_operand:FIXED2 1 "register_operand" "r")))]
  ""
  "* return fract_out (insn, operands, 1, NULL);"
  [(set_attr "cc" "clobber")])

(define_insn "fractuns<FIXED2:mode><FIXED1:mode>2"
  [(set (match_operand:FIXED1 0 "register_operand" "=r")
        (unsigned_fract_convert:FIXED1 (match_operand:FIXED2 1 "register_operand" "r")))]
  ""
  "* return fract_out (insn, operands, 0, NULL);"
  [(set_attr "cc" "clobber")])

;;; Addition/Subtraction, mostly identical to integer versions

(define_insn "add<ALLQQ:mode>3"
  [(set (match_operand:ALLQQ 0 "register_operand" "=r,d")
        (plus:ALLQQ (match_operand:ALLQQ 1 "register_operand" "%0,0")
                    (match_operand:ALLQQ 2 "nonmemory_operand" "r,i")))]
  ""
  "@
        add %0,%2
        subi %0,lo8(-(%2))"
  [(set_attr "length" "1,1")
   (set_attr "cc" "set_czn,set_czn")])

(define_insn "sub<ALLQQ:mode>3"
  [(set (match_operand:ALLQQ 0 "register_operand" "=r,d")
        (minus:ALLQQ (match_operand:ALLQQ 1 "register_operand" "0,0")
                     (match_operand:ALLQQ 2 "nonmemory_operand" "r,i")))]
  ""
  "@
 	sub %0,%2
 	subi %0,lo8(%2)"
  [(set_attr "length" "1,1")
   (set_attr "cc" "set_czn,set_czn")])


(define_insn "add<ALLHQHA:mode>3"
  [(set (match_operand:ALLHQHA 0 "register_operand" "=r,d")
 	(plus:ALLHQHA (match_operand:ALLHQHA 1 "register_operand" "%0,0")
                    (match_operand:ALLHQHA 2 "nonmemory_operand" "r,i")))]
  ""
  "@
 	add %A0,%A2\;adc %B0,%B2
        subi %A0,lo8(-(%2))\;sbci %B0,hi8(-(%2))"
  [(set_attr "length" "2,2")
   (set_attr "cc" "set_n,set_czn")])

(define_insn "sub<ALLHQHA:mode>3"
  [(set (match_operand:ALLHQHA 0 "register_operand" "=r,d")
        (minus:ALLHQHA (match_operand:ALLHQHA 1 "register_operand" "0,0")
		  (match_operand:ALLHQHA 2 "nonmemory_operand" "r,i")))]
  ""
  "@
	sub %A0,%A2\;sbc %B0,%B2
	subi %A0,lo8(%2)\;sbci %B0,hi8(%2)"
  [(set_attr "length" "2,2")
   (set_attr "cc" "set_czn,set_czn")])

(define_insn "add<ALLSA:mode>3"
  [(set (match_operand:ALLSA 0 "register_operand" "=r,d")
 	(plus:ALLSA (match_operand:ALLSA 1 "register_operand" "%0,0")
                    (match_operand:ALLSA 2 "nonmemory_operand" "r,i")))]
  ""
  "@
	add %A0,%A2\;adc %B0,%B2\;adc %C0,%C2\;adc %D0,%D2
	subi %0,lo8(-(%2))\;sbci %B0,hi8(-(%2))\;sbci %C0,hlo8(-(%2))\;sbci %D0,hhi8(-(%2))"
  [(set_attr "length" "4,4")
   (set_attr "cc" "set_n,set_czn")])

(define_insn "sub<ALLSA:mode>3"
  [(set (match_operand:ALLSA 0 "register_operand" "=r,d")
        (minus:ALLSA (match_operand:ALLSA 1 "register_operand" "0,0")
		  (match_operand:ALLSA 2 "nonmemory_operand" "r,i")))]
  ""
  "@
	sub %0,%2\;sbc %B0,%B2\;sbc %C0,%C2\;sbc %D0,%D2
	subi %A0,lo8(%2)\;sbci %B0,hi8(%2)\;sbci %C0,hlo8(%2)\;sbci %D0,hhi8(%2)"
  [(set_attr "length" "4,4")
   (set_attr "cc" "set_czn,set_czn")])

;******************************************************************************
; mul

(define_insn "mulqq3"
  [(set (match_operand:QQ 0 "register_operand" "=r")
	(mult:QQ (match_operand:QQ 1 "register_operand" "a")
		 (match_operand:QQ 2 "register_operand" "a")))]
  "AVR_HAVE_MUL"
  "fmuls %1,%2\;mov %0,r1\;clr r1"
  [(set_attr "length" "3")
   (set_attr "cc" "clobber")])

(define_insn "muluqq3"
  [(set (match_operand:UQQ 0 "register_operand" "=r")
	(mult:UQQ (match_operand:UQQ 1 "register_operand" "r")
                  (match_operand:UQQ 2 "register_operand" "r")))]
  "AVR_HAVE_MUL"
  "mul %1,%2\;mov %0,r1\;clr r1"
  [(set_attr "length" "3")
   (set_attr "cc" "clobber")])

;; (reg:ALLHQ 20) not clobbered on the enhanced core.
;; use registers from 16-23 so we can use fmuls
;; All call-used registers clobbered otherwise - normal library call.
(define_expand "mul<ALLHQ:mode>3"
  [(set (reg:ALLHQ 22) (match_operand:ALLHQ 1 "register_operand" ""))
   (set (reg:ALLHQ 20) (match_operand:ALLHQ 2 "register_operand" ""))
   (parallel [(set (reg:ALLHQ 18) (mult:ALLHQ (reg:ALLHQ 22) (reg:ALLHQ 20)))
	      (clobber (reg:ALLHQ 22))])
   (set (match_operand:ALLHQ 0 "register_operand" "") (reg:ALLHQ 18))]
  "AVR_HAVE_MUL"
  "")

(define_insn "*mul<ALLHQ:mode>3_enh_call"
  [(set (reg:ALLHQ 18) (mult:ALLHQ (reg:ALLHQ 22) (reg:ALLHQ 20)))
   (clobber (reg:ALLHQ 22))]
  "AVR_HAVE_MUL"
  "%~call __mul<ALLHQ:mode>3"
  [(set_attr "type" "xcall")
   (set_attr "cc" "clobber")])

; Special calls for with and without mul.
(define_expand "mul<ALLHA:mode>3"
  [(set (reg:ALLHA 22) (match_operand:ALLHA 1 "register_operand" ""))
   (set (reg:ALLHA 20) (match_operand:ALLHA 2 "register_operand" ""))
   (parallel [(set (reg:ALLHA 18) (mult:ALLHA (reg:ALLHA 22) (reg:ALLHA 20)))
	      (clobber (reg:ALLHA 22))])
   (set (match_operand:ALLHA 0 "register_operand" "") (reg:ALLHA 18))]
  ""
  "
{
  if (!AVR_HAVE_MUL)
    {
      emit_insn (gen_mul<ALLHA:mode>3_call (operands[0], operands[1], operands[2]));
      DONE;
    }
}")

(define_insn "*mul<ALLHA:mode>3_enh"
  [(set (reg:ALLHA 18) (mult:ALLHA (reg:ALLHA 22) (reg:ALLHA 20)))
   (clobber (reg:ALLHA 22))]
  "AVR_HAVE_MUL"
  "%~call __mul<ALLHA:mode>3"
  [(set_attr "type" "xcall")
   (set_attr "cc" "clobber")])

; Without multiplier, clobbers both inputs, and needs a separate output register
(define_expand "mul<ALLHA:mode>3_call"
  [(set (reg:ALLHA 24) (match_operand:ALLHA 1 "register_operand" ""))
   (set (reg:ALLHA 22) (match_operand:ALLHA 2 "register_operand" ""))
   (parallel [(set (reg:ALLHA 18) (mult:ALLHA (reg:ALLHA 22) (reg:ALLHA 24)))
              (clobber (reg:ALLHA 22))
              (clobber (reg:ALLHA 24))])
   (set (match_operand:ALLHA 0 "register_operand" "") (reg:ALLHA 18))]
  "!AVR_HAVE_MUL"
  "")

(define_insn "*mul<ALLHA:mode>3_call"
  [(set (reg:ALLHA 18) (mult:ALLHA (reg:ALLHA 22) (reg:ALLHA 24)))
   (clobber (reg:ALLHA 22))
   (clobber (reg:ALLHA 24))]
  "!AVR_HAVE_MUL"
  "%~call __mul<ALLHA:mode>3"
  [(set_attr "type" "xcall")
   (set_attr "cc" "clobber")])

;; On the enhanced core, don't clobber either input, and use a separate output,
;; r2 is needed as a zero register since r1 is used for mul
(define_expand "mul<ALLSA:mode>3"
  [(set (reg:ALLSA 16) (match_operand:ALLSA 1 "register_operand" ""))
   (set (reg:ALLSA 20) (match_operand:ALLSA 2 "register_operand" ""))
   (parallel [(set (reg:ALLSA 24) (mult:ALLSA (reg:ALLSA 16) (reg:ALLSA 20)))
	      (clobber (reg:QI 15))])
   (set (match_operand:ALLSA 0 "register_operand" "") (reg:ALLSA 24))]
  ""
  "
{
  if (!AVR_HAVE_MUL)
    {
      emit_insn (gen_mul<ALLSA:mode>3_call (operands[0], operands[1], operands[2]));
      DONE;
    }
}")

(define_insn "*mul<ALLSA:mode>3_enh"
  [(set (reg:ALLSA 24) (mult:ALLSA (reg:ALLSA 16) (reg:ALLSA 20)))
   (clobber (reg:QI 15))]
  "AVR_HAVE_MUL"
  "%~call __mul<ALLSA:mode>3"
  [(set_attr "type" "xcall")
   (set_attr "cc" "clobber")])

; Without multiplier, clobbers both inputs, needs a separate output, and also
; needs two more scratch registers
(define_expand "mul<ALLSA:mode>3_call"
  [(set (reg:ALLSA 18) (match_operand:ALLSA 1 "register_operand" ""))
   (set (reg:ALLSA 24) (match_operand:ALLSA 2 "register_operand" ""))
   (parallel [(set (reg:ALLSA 14) (mult:ALLSA (reg:ALLSA 18) (reg:ALLSA 24)))
              (clobber (reg:ALLSA 18))
              (clobber (reg:ALLSA 24))
	      (clobber (reg:HI 22))])
   (set (match_operand:ALLSA 0 "register_operand" "") (reg:ALLSA 14))]
  "!AVR_HAVE_MUL"
  "")

(define_insn "*mul<ALLSA:mode>3_call"
  [(set (reg:ALLSA 14) (mult:ALLSA (reg:ALLSA 18) (reg:ALLSA 24)))
   (clobber (reg:ALLSA 18))
   (clobber (reg:ALLSA 24))
   (clobber (reg:HI 22))]
  "!AVR_HAVE_MUL"
  "%~call __mul<ALLSA:mode>3"
  [(set_attr "type" "xcall")
   (set_attr "cc" "clobber")])

; / / / / / / / / / / / / / / / / / / / / / / / / / / / / / / / / / / / / / /
; div

(define_code_iterator usdiv [udiv div]) ; do signed and unsigned in one shot

(define_expand "<usdiv:code><ALLQQ:mode>3"
  [(set (reg:ALLQQ 25) (match_operand:ALLQQ 1 "register_operand" ""))
   (set (reg:ALLQQ 22) (match_operand:ALLQQ 2 "register_operand" ""))
   (parallel [(set (reg:ALLQQ 24) (usdiv:ALLQQ (reg:ALLQQ 25) (reg:ALLQQ 22)))
	      (clobber (reg:ALLQQ 25))
	      (clobber (reg:QI 23))])
   (set (match_operand:ALLQQ 0 "register_operand" "") (reg:ALLQQ 24))]
  ""
  "")

(define_insn "*<usdiv:code><ALLQQ:mode>3_call"
  [(set (reg:ALLQQ 24) (usdiv:ALLQQ (reg:ALLQQ 25) (reg:ALLQQ 22)))
   (clobber (reg:ALLQQ 25))
   (clobber (reg:QI 23))]
  ""
  "%~call __<usdiv:code><ALLQQ:mode>3"
  [(set_attr "type" "xcall")
   (set_attr "cc" "clobber")])

(define_expand "<usdiv:code><ALLHQHA:mode>3"
  [(set (reg:ALLHQHA 26) (match_operand:ALLHQHA 1 "register_operand" ""))
   (set (reg:ALLHQHA 22) (match_operand:ALLHQHA 2 "register_operand" ""))
   (parallel [(set (reg:ALLHQHA 24) (usdiv:ALLHQHA (reg:ALLHQHA 26) (reg:ALLHQHA 22)))
	      (clobber (reg:ALLHQHA 26))
	      (clobber (reg:QI 21))])
   (set (match_operand:ALLHQHA 0 "register_operand" "") (reg:ALLHQHA 24))]
  ""
  "")

(define_insn "*<usdiv:code><ALLHQHA:mode>3_call"
  [(set (reg:ALLHQHA 24) (usdiv:ALLHQHA (reg:ALLHQHA 26) (reg:ALLHQHA 22)))
   (clobber (reg:ALLHQHA 26))
   (clobber (reg:QI 21))]
  ""
  "%~call __<usdiv:code><ALLHQHA:mode>3"
  [(set_attr "type" "xcall")
   (set_attr "cc" "clobber")])

; note the first parameter gets passed in already offset by 2 bytes
(define_expand "<usdiv:code><ALLSA:mode>3"
  [(set (reg:ALLSA 24) (match_operand:ALLSA 1 "register_operand" ""))
   (set (reg:ALLSA 18) (match_operand:ALLSA 2 "register_operand" ""))
   (parallel [(set (reg:ALLSA 22) (usdiv:ALLSA (reg:ALLSA 24) (reg:ALLSA 18)))
	      (clobber (reg:HI 26))
              (clobber (reg:HI 30))])
   (set (match_operand:ALLSA 0 "register_operand" "") (reg:ALLSA 22))]
  ""
  "")

(define_insn "*<usdiv:code><ALLSA:mode>3_call"
  [(set (reg:ALLSA 22) (usdiv:ALLSA (reg:ALLSA 24) (reg:ALLSA 18)))
   (clobber (reg:HI 26))
   (clobber (reg:HI 30))]
  ""
  "%~call __<usdiv:code><ALLSA:mode>3"
  [(set_attr "type" "xcall")
   (set_attr "cc" "clobber")])


;; abs must be defined for fixed types for correct operation

;; abs(x) abs(x) abs(x) abs(x) abs(x) abs(x) abs(x) abs(x) abs(x) abs(x) abs(x)

;; abs

(define_insn "abs<ALLQQ:mode>2"
  [(set (match_operand:ALLQQ 0 "register_operand" "=r")
        (abs:ALLQQ (match_operand:ALLQQ 1 "register_operand" "0")))]
  ""
  "sbrc %0,7
	neg %0"
  [(set_attr "length" "2")
   (set_attr "cc" "clobber")])
